/**
# Simple Simulation of Evolution
Evolution Strategies, sometimes also
referred to as Evolutionary Strategies, and Evolutionary
Programming are search paradigms
inspired by the principles of biological evolution.
They belong to the family of evolutionary
algorithms that address optimization problems
by implementing a repeated process of
(small) stochastic variations followed by selection:
in each generation (or iteration), new off-
spring (or candidate solutions) are generated
from their parents (candidate solutions already
visited), their fitness is evaluated, and the better
offspring are selected to become the parents for
the next generation.
Evolution strategies most commonly address
the problem of continuous black-box optimization.
The search space is the continuous domain,
solutions in search space are n-dimensional
vectors, and an objective or fitness function is
to be minimized. We make no specific assumptions
on that function, other than that it can be evaluated
for each solution, and refer to this search problem
as black-box optimization. The objective
is, loosely speaking, to generate solutions
with small function-values while using a small
number of function-evaluations.

Run:

$ pub run evo.dart
$ dart bin/evo.dart

*/

library evo;

export './src/utils.dart';
export './src/agent.dart';
export './src/property.dart';
export './src/agentPool.dart';
export './src/environment.dart';
export './src/algorithm.dart';
export './src/crossover.dart';

void main() => print("Library Differential Evolutions");
