import "dart:math" as m;
import "./agent.dart";
import "./utils.dart" as utils;
import "./exception.dart" as exceptions;
import 'property.dart';

//
class Agent {
  Agent(List<Property> this._vector);
  // nur mit Strategie
  factory Agent.createUniformWithStrategy(num center, num strategy, num size) {
    // Der Suchraum sollte zu Iteration bei der Generierung dienen.
    List<Property> vector = new List<PropertyWithStrategy>();
    // for size of search space
    for (var i = 0; i < size; i++) {
      var p = new PropertyWithStrategy(center, strategy);
      vector.add(p);
      print("create ${p}");
    }
    return new Agent(vector);
  }
  // nur mit Strategie
  factory Agent.createFromVectorWithStrategy(
      List<num> center, List<num> strategy) {
    num fl = center.length;
    num sl = strategy.length;

    List<Property> vector = new List<PropertyWithStrategy>();

    if (fl == sl) {
      for (var i = 0; i < sl; i++) {
        var p = new PropertyWithStrategy(0, 1);
        vector.add(p);
      }
    } else {
      throw new exceptions.RuntimeException(
          "Agent.createFromVector - Lengths differ: center ${fl} vs. strategy ${sl}");
    }
    return new Agent(vector);
  }
  // nur mit Strategie
  factory Agent.createNormalWithStrategy(num size) {
    List<Property> vector = new List<PropertyWithStrategy>();
    for (var i = 0; i < size; i++) {
      var p = new PropertyWithStrategy(0, 1);
      vector.add(p);
    }
    return new Agent(vector);
  }
  // egal ob mit Strategie
  factory Agent.createFromCrossover(Agent first, Agent second) {
    num fl = first.length;
    num sl = second.length;

    List<PropertyWithStrategy> cv = new List<PropertyWithStrategy>();
    if (fl == sl) {
      List<PropertyWithStrategy> fv = first.vector;
      List<PropertyWithStrategy> sv = second.vector;
      List<PropertyWithStrategy> cv;
      for (int i = 0; i < fl; i++) {
        PropertyWithStrategy fg = fv[i];
        PropertyWithStrategy sg = sv[i];
        cv.add(new PropertyWithStrategy(
            (fg.value + sg.value) / 2.0, (fg.deviation + sg.deviation) / 2.0));
      }
    } else {
      throw new exceptions.RuntimeException(
          "Crossover - Lengths differ: ${fl} vs ${sl}");
    }
    return new Agent(cv);
  }
  // functional-style Mutation of Agent
  /* 
  factory Agent.createFromMutation(Agent parent) {
    List<PropertyWithStrategy> pv = parent.vector;
    List<PropertyWithStrategy> cv = new  List<PropertyWithStrategy>();
    for (int i = 0; i < pv.length; i++) {
      PropertyWithStrategy pg = pv[i];
      cv.add(new PropertyWithStrategy(pg.value + utils.rdouble() * pg.deviation,
          pg.deviation + utils.rdouble()));

    }
    return new Agent(cv);
  }
  */
  // egal ob mit Strategie
  factory Agent.createFromMutation(Agent parent) {
    var c = new Agent.createAsClone(parent);
    c.mutate();
    return c;
  }
  factory Agent.createAsClone(Agent parent) {
    List<PropertyWithStrategy> pv = parent.vector;
    List<PropertyWithStrategy> cv = new List<PropertyWithStrategy>();
    for (int i = 0; i < pv.length; i++) {
      PropertyWithStrategy pg = pv[i];
      cv.add(new PropertyWithStrategy(pg.value, pg.deviation));
    }
    return new Agent(cv);
  }
  List<Property> _vector;

  List<Property> get vector => this._vector;
  set vector(List<Property> v) => this._vector = v;
  num get length => this._vector.length;

  num fitness = 0;
  num objective_function(List<PropertyWithStrategy> vector) {
    return vector.fold(0.0, (sum, x) => sum + (m.pow(x.value, 2.0)));
  }

  void evaluate() {
    this.fitness = this.objective_function(this.vector);
  }

  void mutate() {
    vector.forEach((Property x) => x.mutate());
  }

  @override
  String toString() {
    return ("Agent " +
        "Fitness: " +
        this.fitness.toString() +
        " Vector: " +
        this.vector.toString());
  }
}
