import 'agent.dart';
import 'property.dart';
import 'utils.dart' show zip2, zip3, randomDouble;

/// A [Crossover] represents
///
abstract class Crossover {
  Agent cross(Agent a1, List<Agent> a2);
}

class CrossoverSimple implements Crossover {
  @override
  Agent cross(Agent a1, List<Agent> a2) {
    assert(a2.length > 0 && a2 != null && a1 != null);
    var lp1 = a1.vector;
    var lp2 = a2[0].vector;
    var res = zip2(lp1, lp2, (a, b) => (a + b) / 2.0);
    return new Agent(res);
  }
}

/// A [CrossoverDE02] represents a Crossover in Differential Evolution
/// that performs crossover in all properties.
///
class CrossoverDE01 implements Crossover {
  @override
  Agent cross(Agent a1, List<Agent> a2) {
    assert(a2.length > 0 && a2 != null && a1 != null);
    var lp1 = a1.vector;
    var lp2 = a2[0].vector;
    var lp3 = a2[1].vector;
    var res = zip3(lp1, lp2, lp3, (a, b, c) => a + ((b - c) / 2.0));
    return new Agent(res);
  }
}

/// A [CrossoverDE02] represents a Crossover in Differential Evolution
/// with crossover probability.
///
class CrossoverDE02 {
  CrossoverDE02(probability) {
    assert(probability > 0.0, probability < 1.0);
    this._probability = probability;
  }

  double _probability;

  Agent crossDE(Agent a1, Agent a2, Agent a3) {
    assert(a2.length > 0 && a2 != null && a1 != null);
    var lp1 = a1.vector;
    var lp2 = a2.vector;
    var lp3 = a3.vector;
    var res = zip3(lp1, lp2, lp3, (a, b, c) {
      if (randomDouble() < _probability) {
        return a + ((b - c) / 2.0);
      } else {
        return a;
      }
    });
    return new Agent(res);
  }
}
